c +++++ Hyperon Decay +++++++++++++++++++++++++++++++++++++++++
c ... common /RIDS/ reaction ID
c
c All		All with KN	  	:  - (with KN)	p1 : black
c KN		KN reaction  	:  1 (formation)	p1 : black
c tnL		none-mesic Lmbda	:  2 (reaction)	p1 : yellow
c tnS		none-mesic Sigma	:  3 (reaction)	p1 : blue
c pi_abs	pion absorption	:  4 (internal)	p1 : cyan
c SLC		Sigma-Lmbda conv 	:  5 (reaction)	p1 : green
c QFL		QF (mesic) Lmbda	:  6 (reaction)	p1 : magenta
c QFS		QF (mesic) Sigma	:  7 (reaction)	p1 : red
c CBG		const. background	:  8 (accidental)	p1 : blue
c
c -All	All without KN  	:  - (without KN)	p2 : black
c -KN		KN reaction		: -1 (KN decay)	p2 : black
c -tnL	none-mesic Lmbda	: -2 (L decay)	p2 : yellow
c -tnS	none-mesic Sigma	: -3 (S decay)	p2 : blue
c -pI_abs	pion absorption	: -4 (external)	p2 : cyan
c -SLC	igma-Lmbda conv	: -5 (L decay)	p2 : green
c -QFL	QF (mesic) Lmbda	: -6 (L decay)	p2 : magenta
c -QFS	QF (mesic) Sigma	: -7 (S decay)	p2 : red
c -pI_mono	pion abs mono	: -8 (external)	p2 : blue
c



      subroutine Lambda_decay(p, v, point, time, RID)
      implicit NONE
c input 
	real p, v(3), point(3), time
	integer RID
c common
      common /masses/ M_k, M_p, M_n, M_L, M_pi, M_pi0, M_Sp, M_S0, M_Sm
      real  M_k, M_p, M_n, M_L, M_pi, M_pi0, M_Sp, M_S0, M_Sm
c ... common /PIDs/
      integer PID_pion, PID_pi0, PID_proton, PID_neutron,
     &	PID_Lambda, PID_SigmaP, PID_SigmaM, PID_Sigma0,
     &	PID_deuteron, PID_triton, PID_helium3
      common /PIDs/ PID_pion, PID_pi0, PID_proton, PID_neutron, 
     &      PID_Lambda, PID_SigmaP, PID_SigmaM, PID_Sigma0,
     &	PID_deuteron, PID_triton, PID_helium3
c ... common /kaon/ for kaon trajectory
      real  kaon_point(3) ! position of the K-reaction
      real  kt_vector(3) ! trajectory of the kaon injection
      real  kt_point(3) ! position of the K-reaction
      real  K_stop_time, K_flight_time, dL_T0
      common /kaon/ kaon_point, kt_point, kt_vector, K_stop_time, K_flight_time, dL_T0
c ... common /RIDS/ reaction ID
	integer KN, tnL, tnS, pi_abs, SLC, QFL, QFS, pi_mono, CBG
	common /RIDS/ KN, tnL, tnS, pi_abs, SLC, QFL, QFS, pi_mono, CBG
c ... common /YPV/ hyperon decay point and vector
	real Y_point(3), Y_v(3), v_dist_L, v_dist_S
	integer Y_ID
      common /YPV/ Y_ID, Y_point, Y_v, v_dist_L, v_dist_S
c local
      real decay_point(3), decay_time, r
      real p_n, v_n(3), p_pi0, v_pi0(3), p_pi, v_pi(3), p_p, v_p(3)
      integer iflag, RID_set
c external
	real rn
c begin
c ... trace Lambda motion in the target
	if(RID .eq. -KN) then
	   RID_set = RID
	else
	   RID_set = -RID
	endif
      decay_point(1) =  point(1)
      decay_point(2) =  point(2)
      decay_point(3) =  point(3)
	decay_time = time
      call  Travel_neutral(p,decay_point,v,PID_Lambda,decay_time)
      Y_point(1) =  decay_point(1)
      Y_point(2) =  decay_point(2)
      Y_point(3) =  decay_point(3)
      Y_v(1) =  decay_point(1) - point(1)
      Y_v(2) =  decay_point(2) - point(2)
      Y_v(3) =  decay_point(3) - point(3)
      r =  sqrt( Y_v(1)**2 + Y_v(2)**2 + Y_v(3)**2 )
	Y_v(1) = Y_v(1) / r
	Y_v(2) = Y_v(2) / r
	Y_v(3) = Y_v(3) / r
	Y_ID = 0
	iflag = 0
c free decay
      if(rn() .le. 0.358) then
c ... L -> n + pi0 decay
         call decay_2_in_flight( M_L, p, v, M_n, M_pi0, p_n, p_pi0, v_n, v_pi0, iflag )
         call put_stack(PID_neutron, p_n, v_n, decay_point, decay_time, RID_set)
         call put_stack(PID_pi0, p_pi0, v_pi0, decay_point, decay_time, RID_set)
      else
c ... L -> p + pi- decay
         call decay_2_in_flight( M_L, p, v, M_p, M_pi, p_p, p_pi, v_p, v_pi, iflag )
         v_dist_L = sqrt((decay_point(1)-kaon_point(1))**2 
     &    	+ (decay_point(2)-kaon_point(2))**2 
     & 	+ (decay_point(3)-kaon_point(3))**2)*10.
         call put_stack(PID_proton, p_p, v_p, decay_point, decay_time, RID_set)
         call put_stack(-PID_pion, p_pi, v_pi, decay_point, decay_time, RID_set)
      endif
      return
      end
        


      subroutine Sigmap_decay(p, v, point, time, RID)
      implicit NONE
c input
      real p, v(3), point(3), time
	integer RID
c common
      common /masses/ M_k, M_p, M_n, M_L, M_pi, M_pi0, M_Sp, M_S0, M_Sm
      real  M_k, M_p, M_n, M_L, M_pi, M_pi0, M_Sp, M_S0, M_Sm
c ... common /PIDs/
      integer PID_pion, PID_pi0, PID_proton, PID_neutron,
     &	PID_Lambda, PID_SigmaP, PID_SigmaM, PID_Sigma0,
     &	PID_deuteron, PID_triton, PID_helium3
      common /PIDs/ PID_pion, PID_pi0, PID_proton, PID_neutron, 
     &      PID_Lambda, PID_SigmaP, PID_SigmaM, PID_Sigma0,
     &	PID_deuteron, PID_triton, PID_helium3
c ... common /RIDS/ reaction ID
	integer KN, tnL, tnS, pi_abs, SLC, QFL, QFS, pi_mono, CBG
	common /RIDS/ KN, tnL, tnS, pi_abs, SLC, QFL, QFS, pi_mono, CBG
c ... common /kaon/ for kaon trajectory
      real  kaon_point(3) ! position of the K-reaction
      real  kt_vector(3) ! trajectory of the kaon injection
      real  kt_point(3) ! position of the K-reaction
      real  K_stop_time, K_flight_time, dL_T0
      common /kaon/ kaon_point, kt_point, kt_vector, K_stop_time, K_flight_time, dL_T0
c ... common /setup/ for simulation run-condition, counter resolution and its efficiency
	integer FF_knucl_decay, multi_set, k_decay_set
	logical Kstop_cor
      real  delta_x, delta_y, delta_T0, delta_NC, TC_eff, T0_eff
      common /setup/ delta_x, delta_y, delta_T0, delta_NC, TC_eff, T0_eff,
     &	FF_knucl_decay, multi_set, k_decay_set, Kstop_cor
c ... common /YPV/ hyperon decay point and vector
	real Y_point(3), Y_v(3), v_dist_L, v_dist_S
	integer Y_ID
      common /YPV/ Y_ID, Y_point, Y_v, v_dist_L, v_dist_S
c ... /FKS/ to simulate without enegy loss of charged Sigma
	logical FakeS
	common /FKS/ FakeS
c local
      real p_p, v_p(3), p_n, v_n(3), p_pi, v_pi(3), p_pi0, v_pi0(3)
	real decay_point(3), decay_time, r
      integer stopbox, init, react, status, decay, multi, fluct, iflag, RID_set
c external
	real rn
c begin
	if(RID .eq. -KN) then
	   RID_set = RID
	else
	   RID_set = -RID
	endif
c ... trace S+ motion in the target
      decay_point(1) = point(1)
      decay_point(2) = point(2)
      decay_point(3) = point(3)
	decay_time = time
      init = 0
      react = 0
      decay = 1
      multi = multi_set
      fluct = 1
	if(FakeS) then
         call  Travel_neutral(p,decay_point,v,PID_SigmaP,decay_time)
	else
         call Travel(p, decay_point,v,PID_SigmaP,StopBox,decay_time,
     &        status, init, react, decay, multi, fluct)
	endif
      Y_point(1) =  decay_point(1)
      Y_point(2) =  decay_point(2)
      Y_point(3) =  decay_point(3)
      Y_v(1) =  decay_point(1) - point(1)
      Y_v(2) =  decay_point(2) - point(2)
      Y_v(3) =  decay_point(3) - point(3)
      r =  sqrt( Y_v(1)**2 + Y_v(2)**2 + Y_v(3)**2 )
	Y_v(1) = Y_v(1) /   r
	Y_v(2) = Y_v(2) /   r
	Y_v(3) = Y_v(3) /   r
	Y_ID = 1
	iflag = 0
c free decay
      if(rn() .le. 0.4831) then
c ... S+ -> n + pi+ decay
         v_dist_S = sqrt((decay_point(1)-kaon_point(1))**2 
     &    	+ (decay_point(2)-kaon_point(2))**2 
     & 	+ (decay_point(3)-kaon_point(3))**2)*10.
         call decay_2_in_flight( M_Sp, p, v, M_n, M_pi, p_n, p_pi, v_n, v_pi, iflag )
         call put_stack(PID_neutron, p_n, v_n, decay_point, decay_time, RID_set)
         call put_stack(PID_pion, p_pi, v_pi, decay_point, decay_time, RID_set)
      else 
c ... S+ -> p + pi0 decay
         call decay_2_in_flight( M_Sp, p, v, M_p, M_pi0, p_p, p_pi0, v_p, v_pi0, iflag )
         call put_stack(PID_proton, p_p, v_p, decay_point, decay_time, RID_set)
         call put_stack(PID_pi0, p_pi0, v_pi0, decay_point, decay_time, RID_set)
      endif
      return
      end
        


      subroutine Sigmam_decay(p, v, point, time, RID)
      implicit NONE
c input
	real p, v(3), point(3), time
	integer RID
c common
      common /masses/ M_k, M_p, M_n, M_L, M_pi, M_pi0, M_Sp, M_S0, M_Sm
      real  M_k, M_p, M_n, M_L, M_pi, M_pi0, M_Sp, M_S0, M_Sm
      real  M_helium4, M_helium3, M_triton, M_deuteron, M_nn
      common /masses2/  M_helium4, M_helium3, M_triton, M_deuteron, M_nn
c ... common /PIDs/
      integer PID_pion, PID_pi0, PID_proton, PID_neutron,
     &	PID_Lambda, PID_SigmaP, PID_SigmaM, PID_Sigma0,
     &	PID_deuteron, PID_triton, PID_helium3
      common /PIDs/ PID_pion, PID_pi0, PID_proton, PID_neutron, 
     &      PID_Lambda, PID_SigmaP, PID_SigmaM, PID_Sigma0,
     &	PID_deuteron, PID_triton, PID_helium3
c ... common /branch_1N_2N/ for single nucleon reaction ratio
	real  r_1N_2N_S, r_1N_2N_L, r_1N_2N_SLC,
     &	r_1N_2N_SNS, r_1N_2N_LNS, r_1N_2N_piS,
     &	r_2N_3N_K2NS, r_2N_3N_K2NL, r_1N_2N_piEx_abs
	common /branch_1N_2N/ r_1N_2N_S, r_1N_2N_L, r_1N_2N_SLC,
     &	r_1N_2N_SNS, r_1N_2N_LNS, r_1N_2N_piS,
     &	r_2N_3N_K2NS, r_2N_3N_K2NL, r_1N_2N_piEx_abs
c ... common /RIDS/ reaction ID
	integer KN, tnL, tnS, pi_abs, SLC, QFL, QFS, pi_mono, CBG
	common /RIDS/ KN, tnL, tnS, pi_abs, SLC, QFL, QFS, pi_mono, CBG
c ... common /kaon/ for kaon trajectory
      real  kaon_point(3) ! position of the K-reaction
      real  kt_vector(3) ! trajectory of the kaon injection
      real  kt_point(3) ! position of the K-reaction
      real  K_stop_time, K_flight_time, dL_T0
      common /kaon/ kaon_point, kt_point, kt_vector, K_stop_time, K_flight_time, dL_T0
c ... common /setup/ for simulation run-condition, counter resolution and its efficiency
	integer FF_knucl_decay, multi_set, k_decay_set
	logical Kstop_cor
      real  delta_x, delta_y, delta_T0, delta_NC, TC_eff, T0_eff
      common /setup/ delta_x, delta_y, delta_T0, delta_NC, TC_eff, T0_eff,
     &	FF_knucl_decay, multi_set, k_decay_set, Kstop_cor
c ... common /YPV/ hyperon decay point and vector
	real Y_point(3), Y_v(3), v_dist_L, v_dist_S
	integer Y_ID
      common /YPV/ Y_ID, Y_point, Y_v, v_dist_L, v_dist_S
c ... /FKS/ to simulate without enegy loss of charged Sigma
	logical FakeS
	common /FKS/ FakeS
c local
      real decay_point(3), decay_time, r
      real p_n2, v_n2(3), p_deuteron, v_deuteron(3)
      real p_n, v_n(3), p_pi, v_pi(3), p_L, v_L(3), p_triton, v_triton(3)
      integer stopbox, init, react, status, decay, multi, fluct
      integer N_nucl, N_pickup, iflag, RID_set
c external
	real rn
c begin
	if(RID .eq. -KN) then
	   RID_set = RID
	else
	   RID_set = -RID
	endif
c ... trace S- motion in the target
      decay_point(1) = point(1)
      decay_point(2) = point(2)
      decay_point(3) = point(3)
	decay_time = time
      init = 0
      react = 0
      decay = 1
      multi = multi_set
      fluct = 1
	if(FakeS) then
         call  Travel_neutral(p,decay_point,v,PID_SigmaM,decay_time)
	   status = 0
	else
         call Travel(p,decay_point,v,PID_SigmaM,StopBox,decay_time,
     &        status, init, react, decay, multi, fluct)
	endif
      Y_point(1) =  decay_point(1)
      Y_point(2) =  decay_point(2)
      Y_point(3) =  decay_point(3)
      Y_v(1) =  decay_point(1) - point(1)
      Y_v(2) =  decay_point(2) - point(2)
      Y_v(3) =  decay_point(3) - point(3)
      r =  sqrt( Y_v(1)**2 + Y_v(2)**2 + Y_v(3)**2 )
	Y_v(1) = Y_v(1) /   r
	Y_v(2) = Y_v(2) /   r
	Y_v(3) = Y_v(3) /   r
	Y_ID = 2
      if(status .eq. -1 .and. rn() .lt.0.97) then
c S- stop in the target : external conversion occurs
         if(rn() .lt. r_1N_2N_SLC) then 
            N_nucl = 4
            N_pickup = 1 
            iflag = 0
            call reaction_at_rest_3( M_helium4, M_Sm, 
     &            M_L, M_n, M_triton,
     &            p_L, p_n, p_triton, 
     &            v_L, v_n, v_triton, N_nucl, N_pickup, iflag)
            call put_stack(PID_neutron, p_n, v_n, decay_point, decay_time, SLC)
            call Lambda_decay(p_L, v_L, decay_point, decay_time, SLC)
         else
            N_nucl = 4
            N_pickup = 1 
            iflag = 0
            call breakup_at_rest_4( M_helium4, M_Sm, 
     &            M_L, M_n, M_n,  M_deuteron,
     &            p_L, p_n, p_n2, p_deuteron, 
     &            v_L, v_n, v_n2, v_deuteron, N_nucl, N_pickup, iflag)
            call put_stack(PID_neutron, p_n, v_n, decay_point, decay_time, SLC)
            call put_stack(PID_neutron, p_n2, v_n2, decay_point, decay_time, SLC)
            call Lambda_decay(p_L, v_L, decay_point, decay_time, SLC)
         endif
      else
c free decay
c ... S- -> n pi- 100%
	   iflag = 0
         v_dist_S = sqrt((decay_point(1)-kaon_point(1))**2 
     &    	+ (decay_point(2)-kaon_point(2))**2 
     & 	+ (decay_point(3)-kaon_point(3))**2)*10.
         call decay_2_in_flight( M_Sm, p, v, M_n, M_pi, p_n, p_pi, v_n, v_pi, iflag )
         call put_stack(PID_neutron, p_n, v_n, decay_point, decay_time, RID_set)
         call put_stack(-PID_pion, p_pi, v_pi, decay_point, decay_time, RID_set)
      endif
      return
      end



      subroutine Sigma0_decay(p, v, point, time, RID)
      implicit NONE
c input
	real p, v(3), point(3), time
	integer RID
c local
      integer iflag
      real decay_time, decay_point(3)
      real M_g, p_L, v_L(3), p_g, v_g(3)
c ... common for mass
      common /masses/ M_k, M_p, M_n, M_L, M_pi, M_pi0, M_Sp, M_S0, M_Sm
      real  M_k, M_p, M_n, M_L, M_pi, M_pi0, M_Sp, M_S0, M_Sm
c begin
c free decay
c ... S0 -> L + gamma decay
      decay_point(1) = point(1)
      decay_point(2) = point(2)
      decay_point(3) = point(3)
	decay_time = time
         M_g = 0.
         call decay_2_in_flight( M_S0, p, v, 
     &		M_L, m_g, p_L, p_g, v_L, v_g, iflag )
         call Lambda_decay(p_L, v_L, decay_point, decay_time, RID)
      return
      end





      subroutine Travel_neutral(Momentum,point,vector,particle,time)
      implicit NONE
      real  momentum, point(3), vector(3), time
      integer particle
      real  Mass,tau, rn
      integer Zi
c local 
      real  hx, ct
      integer j
c begin
      call  PID_get(particle,Mass,tau,Zi)
      ct =  -tau*log(rn()+1.e-10)
      hx =  Momentum/Mass * ct
      time  = time + ct / 29.9792458 * sqrt(Momentum**2 + Mass**2) / Mass
      do j  = 1, 3
         point(j) = point(j)  + hx * vector(j)
      enddo
      return
      end





